package com.sky.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;

@Slf4j
@AllArgsConstructor
@Data
public class FileBrowserUtil {
    private String domain;
    private String username;
    private String password;

    /**
     * —— 第一步：登录拿 token ——
     * 调用 /api/login 接口，返回纯 JWT 字符串，或 {"token":"..."} 结构
     */
    public String login() throws IOException {
        String url = domain + "/api/login";
        try (CloseableHttpClient client = HttpClients.createDefault()) {
            HttpPost post = new HttpPost(url);
            post.setHeader("Content-Type", "application/json");

            // 构造登录参数
            JSONObject cred = new JSONObject();
            cred.put("username", username);
            cred.put("password", password);
            post.setEntity(new StringEntity(cred.toString(), StandardCharsets.UTF_8));

            try (CloseableHttpResponse resp = client.execute(post)) {
                int status = resp.getStatusLine().getStatusCode();
                String body = EntityUtils.toString(resp.getEntity(), StandardCharsets.UTF_8).trim();

                if (status >= 200 && status < 300) {
                    // 如果返回 JSON 对象，则解析出 token 字段
                    if (body.startsWith("{") && body.endsWith("}")) {
                        JSONObject obj = JSONObject.parseObject(body);
                        String token = obj.getString("token");
                        log.info("登录成功，token={}", token);
                        return token;
                    }
                    // 否则直接当成原始 JWT 返回
                    log.info("登录成功，token={}", body);
                    return body;
                } else {
                    log.error("Login failed: HTTP {} - {}", status, body);
                    throw new IOException("Login failed: HTTP " + status);
                }
            }
        }
    }

    /**
     * —— 第二步：上传文件 ——
     * POST {domain}/api/resources/{encodedPath}?override=true
     * Header: X-Auth: token
     */
    public String uploadFile(byte[] fileBytes, String fileName) throws IOException {
        String token = login();
        String remotePath = "store/" + fileName;
        String encodedPath = URLEncoder
                .encode(remotePath, StandardCharsets.UTF_8)
                .replace("%2F", "/");

        // 根据后缀猜 MIME 类型
        String mimeType = URLConnection.guessContentTypeFromName(fileName);
        if (mimeType == null) {
            mimeType = "application/octet-stream";
        }

        String uploadUrl = domain + "/api/resources/" + encodedPath + "?override=true";
        try (CloseableHttpClient client = HttpClients.createDefault()) {
            HttpPost post = new HttpPost(uploadUrl);
            post.setHeader("X-Auth", token);
            post.setEntity(new ByteArrayEntity(fileBytes, ContentType.create(mimeType)));

            try (CloseableHttpResponse resp = client.execute(post)) {
                int status = resp.getStatusLine().getStatusCode();
                String respBody = EntityUtils.toString(resp.getEntity(), StandardCharsets.UTF_8);
                if (status < 200 || status >= 300) {
                    log.error("文件上传失败: HTTP {} - {}", status, respBody);
                    throw new IOException("Upload failed: HTTP " + status);
                }
                log.info("文件上传成功，remotePath={}", remotePath);
                return remotePath;
            }
        }
    }

    /**
     * 第三步：生成公开分享链接
     * 模拟浏览器的 POST /api/share/{encodedPath} 请求，body 为 "{}"
     */
    public String createShareLink(String remotePath) throws IOException {
        String token = login();

        // URL encode 并保留斜杠
        String encodedPath = URLEncoder
                .encode(remotePath, StandardCharsets.UTF_8)
                .replace("%2F", "/");

        String shareUrl = domain + "/api/share/" + encodedPath;
        log.info("准备创建分享链接，POST {}", shareUrl);

        try (CloseableHttpClient client = HttpClients.createDefault()) {
            HttpPost post = new HttpPost(shareUrl);
            post.setHeader("Cookie", "auth=" + token);
            post.setHeader("X-Auth", token);
            post.setHeader("Content-Type", "text/plain;charset=UTF-8");
            post.setEntity(new StringEntity("{}", StandardCharsets.UTF_8));

            try (CloseableHttpResponse resp = client.execute(post)) {
                int status = resp.getStatusLine().getStatusCode();
                String body = EntityUtils.toString(resp.getEntity(), StandardCharsets.UTF_8).trim();

                if (status < 200 || status >= 300) {
                    log.error("创建分享失败 HTTP {} - {}", status, body);
                    throw new IOException("Share failed: HTTP " + status);
                }

                // ========= 这里改为先检测是对象还是数组 =========
                JSONObject json;
                if (body.startsWith("[")) {
                    // 如果真返回数组（老版本可能是 [{...}]）
                    json = JSONArray.parseArray(body).getJSONObject(0);
                } else if (body.startsWith("{")) {
                    // 当前版本直接返回对象
                    json = JSONObject.parseObject(body);
                } else {
                    throw new IOException("Unexpected share response: " + body);
                }

                String hash = json.getString("hash");
                String publicUrl = domain + "/api/public/dl/" + hash + "/" + remotePath;
                log.info("创建分享链接成功：{}", publicUrl);
                return publicUrl;
            }
        }
    }

    /**
     * —— 整合示例：上传并立即返回分享链接 ——
     */
    public String uploadAndGetUrl(byte[] fileBytes, String fileName) throws IOException {
        String remotePath = uploadFile(fileBytes, fileName);
        return createShareLink(remotePath);
    }
}
