package com.sky.controller.admin;

import com.sky.dto.DishDTO;
import com.sky.dto.DishPageQueryDTO;
import com.sky.entity.Dish;
import com.sky.result.PageResult;
import com.sky.result.Result;
import com.sky.service.DishService;
import com.sky.vo.DishVO;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController("adminDishController")
@RequestMapping("/admin/dish")
@Slf4j
public class DishController {

    @Autowired
    private DishService dishService;

    @PostMapping
    @ApiOperation("新增菜品")
    // 新增后清除当前分类的缓存  ，注意这里缓存的是菜品分类，每个分类中包括多个菜品
    @CacheEvict(value = "dishCache", key = "'dish_' + #dishDTO.categoryId")
    public Result<Void> save(@RequestBody DishDTO dishDTO){
        log.info("新增菜品: {}", dishDTO);
        dishService.saveWithFlavor(dishDTO);
        return Result.success();
    }

    @GetMapping("/page")
    @ApiOperation("菜品分页查询")
    public Result<PageResult> page(DishPageQueryDTO dishPageQueryDTO){
        PageResult pageResult = dishService.pageQuery(dishPageQueryDTO);
        return Result.success(pageResult);
    }

    @DeleteMapping
    @ApiOperation("菜品批量删除")
    // 删除后清除所有分类下的菜品缓存
    @CacheEvict(value = "dishCache", allEntries = true)
    public Result<Void> delete(@RequestParam List<Long> ids){
        log.info("菜品批量删除：{}", ids);
        dishService.deleteBatch(ids);
        return Result.success();
    }

    @GetMapping("/{id}")
    @ApiOperation("根据id查询菜品及口味信息")
    public Result<DishVO> queryByIdWithFlavor(@PathVariable Long id){
        DishVO dishVO = dishService.getByIdWithFlavor(id);
        return Result.success(dishVO);
    }

    @PutMapping
    @ApiOperation("更新菜品")
    // 更新后清除所有分类下的菜品缓存，保证下一次查询是最新数据
    @CacheEvict(value = "dishCache", allEntries = true)
    public Result<Void> update(@RequestBody DishDTO dishDTO){
        log.info("更新菜品：{}", dishDTO);
        dishService.updateWithFlavor(dishDTO);
        return Result.success();
    }

    @PostMapping("/status/{status}")
    @ApiOperation("菜品起售/停售")
    // 改变状态后也清空所有分类缓存
    @CacheEvict(value = "dishCache", allEntries = true)
    public Result<Void> startOrStop(@PathVariable("status") Integer status,
                                    @RequestParam Long id){
        dishService.startOrStop(status, id);
        return Result.success();
    }

    @GetMapping("/list")
    @ApiOperation("根据分类id查询菜品")
    @Cacheable(value = "dishCache", key = "'dish_' + #categoryId")
    public Result<List<Dish>> list(@RequestParam("categoryId") Long categoryId) {
        // Spring Cache 会自动：
        // 1. 先查缓存 dishCache::dish_{categoryId}
        // 2. 没有才执行方法体，并把返回值缓存
        List<Dish> list = dishService.list(categoryId);
        return Result.success(list);
    }
}
